# Platform.io with IDF seemed to have some shortcomings:
#  1. No possibility to configure two different sdkconfig.defaults for different
#     environments.
#  2. No possibility to use platform.io extends option to quickly plug together
#     hardware and software settings.
#
# For (1), we assume the platfrom.io settings custom_idfSdkConfigDefaults and
# custom_idfSdkConfig that provie the patch to sdkconfig fiels, relative to the
# project root folder. It will write the sdkconfig.defaults into the project
# main dir, replacing any existing file and remove the sdkconfig.<environment>
# to ensure the config is applied. If both parameters are applied, it will merge
# the settings into one file.
#
# For (2), the setting custom_idfSdkCMakeLists is used like above.
#
# The script does traverse one level of 'extends', but not multiple.

# # Example platformio.ini:
# # Common Configuration
# # ====================
# [env]
# extra_scripts = pre:config/pioEnvHacksForIdf.py
# monitor_speed = 115200
# 
# # Hardware configurations
# # -----------------------
# [esp32Solo]
# platform = espressif32
# board = esp32dev
# framework = arduino, espidf
# custom_idfSdkConfigDefaults = config/idf/sdkconfig.defaults
# custom_idfSdkConfig         = config/idf/sdkconfig.esp32Solo
# [esp32Dual]
# platform = espressif32
# board = esp32dev
# framework = arduino, espidf
# custom_idfSdkConfigDefaults = config/idf/sdkconfig.defaults
# custom_idfSdkConfig         = config/idf/sdkconfig.esp32Dual
# 
# # Software configurations
# # -----------------------
# [devTest]
# build_src_filter = +<mainDevTest.cpp>
# custom_idfCMakeLists = config/idf/CMakeLists.DevTest
# [scaleHmi]
# build_src_filter = +<main.ino>
# custom_idfCMakeLists = config/idf/CMakeLists.ScaleHmi
# 
# # Real Targets (hardware + software)
# # ============
# [env:esp32Solo_devTest]
# extends = esp32Solo, devTest
# [env:esp32Dual_devTest]
# extends = esp32Dual, devTest
# 
# [env:esp32Solo_scaleHmi]
# extends = esp32Solo, scaleHmi
# [env:esp32Dual_scaleHmi]
# extends = esp32Dual, scaleHmi

# This script needs to survive the following use cases:
#  1. Options from sdkconfig and CMakeLists.txt are applied (for the first
#     build)
#  2. after two environments are built, switching between them does not need to
# rebuild For the following use cases, the workaround would be "clean".
#  3. Changes in the sdkconfig files are detected and applied
#  4. Changes in the CMakeLists files are detected and applied Results: all
# good. (3) does only rebuild the main binary while (4) rebuilds everything.


import os
import shutil

Import('env')

# Check input
print('==POST-SCRIPT: pioEnvHacksForIdf.py >> started..')
envName = env['PIOENV']
print('  Environment:', envName)
extendList = env.GetProjectOption('extends')
print('    ..it extends: ', extendList)
config = env.GetProjectConfig()

# manually traverse the 'extends' environments to get the custom options
idfSdkConfigDefaults = ''
idfSdkConfig = ''
idfCMakeLists = ''
for iEnv in (extendList + [envName]):
    print('  processing ' + iEnv)
    if config.has_section(iEnv) and config.has_option(iEnv, 'custom_idfSdkConfigDefaults'):
        idfSdkConfigDefaults = config.get(iEnv, 'custom_idfSdkConfigDefaults')
    if config.has_section(iEnv) and config.has_option(iEnv, 'custom_idfSdkConfig'):
        idfSdkConfig         = config.get(iEnv, 'custom_idfSdkConfig')
    if config.has_section(iEnv) and config.has_option(iEnv, 'custom_idfCMakeLists'):
        idfCMakeLists        = config.get(iEnv, 'custom_idfCMakeLists')
print('  custom_idfSdkConfigDefaults: ', idfSdkConfigDefaults)
print('  custom_idfSdkConfig: '        , idfSdkConfig)
print('  custom_idfCMakeLists: '       , idfCMakeLists)

# Apply sdkconfig.defaults
# ========================
# get defaults
sdkConfig = ''
if (idfSdkConfigDefaults):
    with open(os.path.join(env['PROJECT_DIR'], idfSdkConfigDefaults)) as file:
        sdkConfig = file.read()
# get config and append
if (idfSdkConfig):
    if (sdkConfig):
        sdkConfig += '\n'
    with open(os.path.join(env['PROJECT_DIR'], idfSdkConfig)) as file:
        sdkConfig += file.read()
# debug print
print('  SDKCONFIG:\n', sdkConfig)
# place as sdkconfig.defaults
sdkConfigDefault = os.path.join(env['PROJECT_DIR'], 'sdkconfig.defaults')
#print('  ', sdkConfigDefault)
if (sdkConfig):
    with open(sdkConfigDefault, 'w') as file:
        file.write(sdkConfig)
# cleanup existing sdkconfig.<env> (otherwise, options will not apply)
sdkConfigForEnv = os.path.join(env['PROJECT_DIR'], 'sdkconfig.' + envName)
#print('  ', sdkConfigForEnv)
if (sdkConfig and os.path.exists(sdkConfigForEnv)):
    os.remove(sdkConfigForEnv)

# Apply CMakeLists
# ================
if(idfCMakeLists):
    shutil.copyfile(os.path.join(env['PROJECT_DIR'], idfCMakeLists),
                    os.path.join(env['PROJECT_SRC_DIR'], 'CMakeLists.txt'))

print('==POST-SCRIPT: pioEnvHacksForIdf.py << done.')
